/****************************************************************************
 * tx.argvfix
 *   Conversion of the argv argument of main function to UTF-8.
 *
 *   By default, it is supplied in environment-native character sets,
 *   so it needs a conversion. Also this module remedies the problem
 *   that the runtime of dmd-windows does not correctly parse multibyte
 *   characters. Following one function is provided:
 *
 *     void argvfix( inout char[][] argv )
 *
 *   and is meant to be called at the begenning of the main function.
 ****************************************************************************/
module tx.argvfix;

version(Windows)
	version(DigitalMars)
		version = argvBroken;

version(argvBroken)
{
	import std.utf;
	import std.ctype;
	extern(Windows) wchar* GetCommandLineW();
	extern(C) int wcslen(wchar *);

	void argvfix( inout string[] argv )
	{
		argv.length = 0;

		wchar* wcp = GetCommandLineW();
		string str = wcp[0..wcslen(wcp)].toUTF8();

		const(char)* p = str.ptr;
		const(char)* e = p + str.length;

		bool sp()
		{
			while(p<e && isspace(*p))
				++p;
			return p<e;
		}

		while( sp() )
		{
			char[] arg;

			bool dq = false;
			while( p < e )
				if( dq && *p =='\\' )
					if( p+1 == e ) { arg~=*p++; break;}
					else if( p[1]=='"' ) { arg~=p[1]; p+=2; }
					else { arg~=*p++; }
				else if( *p == '"' )
					{dq=!dq; p++;}
				else if( !dq && isspace(*p) )
					break;
				else
					arg~=*p++;

			argv ~= arg;
		}
	}
}

else
{
	import tx.charset;

	void argvfix( inout string[] argv )
	{
		foreach( inout arg ; argv )
			arg = NativeCharset().toUTF8(arg);
	}
}
